--
-- GroundModification
--
--
-- @author jodamm
-- @date 10/04/2017


GroundModification = {};

local GroundModification_path = g_currentModDirectory;
local GroundModification_dir = string.match(g_currentModDirectory, '/mods/(.*)');

if SpecializationUtil.specializations['GMTracks'] == nil then
    SpecializationUtil.registerSpecialization('GMTracks', 'GMTracks', g_currentModDirectory .. 'GMTracks.lua')
end
if SpecializationUtil.specializations['GroundModifier'] == nil then
    SpecializationUtil.registerSpecialization('GroundModifier', 'GroundModifier', g_currentModDirectory .. 'GroundModifier.lua')
end

function GroundModification:loadMap(savegame)
        for _, vehicle in pairs(VehicleTypeUtil.vehicleTypes) do
            if vehicle ~= nil then
                table.insert(vehicle.specializations, 1, SpecializationUtil.getSpecialization("GMTracks"))
                if SpecializationUtil.hasSpecialization(Roller, vehicle.specializations) then
                    table.insert(vehicle.specializations, SpecializationUtil.getSpecialization("GroundModifier"));
                end
            end
        end
    self.GroundModification_firstRun = true;
    self.layerId = nil;
    self.mapXMLFilename = nil;
    self.mapXMLId = nil;
    self.fieldSound = nil;
    self.newDensityMap = true; --funktion immer aktiv
end

function GroundModification:deleteMap()
end

function GroundModification:mouseEvent(posX, posY, isDown, isUp, button)
end

function GroundModification:keyEvent(unicode, sym, modifier, isDown)
end

function GroundModification:update(dt)
    if self.GroundModification_firstRun == true then
		    self.GroundModification_firstRun = false;
    self.terrainDetailId = g_currentMission.terrainDetailId;
        
        local density_file = "";
        local mapxml_file = "";
        local mapsize = getDensityMapSize(g_currentMission.terrainDetailId);
        
        if g_currentMission.missionInfo.savegameDirectory == nil then
            self.newDensityMap = true; 
            density_file = GroundModification_dir .."density" .. mapsize ..".png";           
            mapxml_file = g_modsDirectory .."groundmodification_map.xml";
            self.createMapXMLFile(mapxml_file, density_file,"");
--            density_file = g_modsDirectory .."groundmodification_density.gdm"
        else
            density_file = g_currentMission.missionInfo.savegameDirectory .."/groundmodification_density.gdm";
            mapxml_file = g_currentMission.missionInfo.savegameDirectory .."/groundmodification_map.xml";
            local density_file_png = g_currentMission.missionInfo.savegameDirectory .."/groundmodification_density.png";

            if not fileExists(density_file) and not fileExists(density_file_png) then
                self.newDensityMap = true;
                density_file_png = "../mods/" ..GroundModification_dir .."density" .. mapsize ..".png";
            else
                density_file_png = "groundmodification_density.png";            
            end
--            if not fileExists(mapxml_file) then
--                print("-----No mapxml file. Creating a new one.");
                self.createMapXMLFile(mapxml_file, density_file_png, "../mods/");            
--           end
        end


        if g_currentMission.surfaceNameToSurfaceSound ~= nil then
            self.fieldSound = g_currentMission.surfaceNameToSurfaceSound.field;
        end
        self.mapXMLFilename = mapxml_file;
        self.mapXMLId = loadXMLFile("GM_mapXML", mapxml_file);
        GroundModification.layerId = loadI3DFile(mapxml_file, false, false, true);
        GroundModification.terrDetail = getChildAt(getChildAt(GroundModification.layerId,0),256);
        -- densityMap mit cultivator density map abgleichen
        if self.newDensityMap then
            local terrid = g_currentMission.terrainDetailId;
            local groundid = GroundModification.terrDetail;            
            setDensityMaskedParallelogram(groundid, mapsize* -0.5, mapsize * -0.5, mapsize, 0, 0, mapsize, 0, 12, terrid, g_currentMission.terrainDetailTypeFirstChannel, g_currentMission.terrainDetailTypeNumChannels, 2048);
--            saveDensityMapToFile(groundid, density_file);            
        end
        
        unlink(GroundModification.terrDetail);
        link(getParent(g_currentMission.terrainDetailId), GroundModification.terrDetail);

    end
end

function GroundModification:draw()
end

function GroundModification.createMapXMLFile(xmlfilename, gdmfilename, relPath)
    local pathMod = relPath .. GroundModification_dir;
    local xmlfn = loadXMLFile("GroundModificationMap" , GroundModification_path .."emptymap.xml"); --load template file
    --get height png
    local map_dir = g_currentMission.missionInfo.map.baseDirectory;
    local mapxml = loadXMLFile("GM_mapxml", map_dir .. g_currentMission.missionInfo.mapXmlFilename);
--    print("----- Open Map XML :" ..map_dir .. g_currentMission.missionInfo.mapXmlFilename); 
    local i3dfile = getXMLString(mapxml, "map.filename");
    delete(mapxml);
    if string.sub(i3dfile,1,1)=="$" then
        i3dfile = string.sub(i3dfile,2,string.len(i3dfile));
    end
--    print("----- Open Map i3d file :" ..map_dir .. i3dfile); 
    local mapi3d = loadXMLFile("GM_mapi3d", map_dir .. i3dfile);
    local demFileId = getXMLInt(mapi3d, "i3D.scene.TerrainTransformGroup#heightMapId");
    local demFilename = "";

    local pathSub2 = "";

    for pathSub in string.gmatch(g_currentMission.missionInfo.mapXmlFilename, '([^/]+)') do
        map_dir = map_dir .. pathSub2;
        pathSub2 = pathSub .. "/";
    end
    if g_currentMission.missionInfo.map.isModMap == false then
        setXMLBool(xmlfn, "i3D.Files.File(0)#relativePath" , false);
    else
        map_dir = relPath .. string.match(map_dir, '/mods/(.*)');
    end
    
    local i = 0;
    while true do
        local key = string.format("i3D.Files.File(%d)", i);
        if not hasXMLProperty(mapi3d, key) then
            break;
        end
        if getXMLInt(mapi3d, key.."#fileId") == demFileId then
            demFilename = getXMLString(mapi3d, key.."#filename");
            if getXMLBool(mapi3d, key.."#relativePath") == true then
                demFilename = map_dir .. demFilename;
                                
            end
            break;
        end     
        i = i + 1;                   
    end    
    delete(mapi3d);
    setXMLString(xmlfn, "i3D.Files.File(0)#filename" , demFilename);
    setXMLString(xmlfn, "i3D.Files.File(1)#filename" , gdmfilename);

    for i=6, 10 do  --todo Flexibel gestalten 
        local fn = getXMLString(xmlfn, string.format("i3D.Files.File(%d)#filename", i));
        fn = pathMod .. fn;
        setXMLString(xmlfn, string.format("i3D.Files.File(%d)#filename", i) , fn);    
    end
    saveXMLFileTo(xmlfn, xmlfilename);
end

function GroundModification.saveSavegame(self, missionInfo)
    local map_file = missionInfo.savegameDirectory .."/groundmodification_map.xml"
    local density_file = missionInfo.savegameDirectory .."/groundmodification_density.gdm";
    if map_file == GroundModification.mapXMLFilename then
        saveXMLFile(GroundModification.mapXMLId);
    else
        saveXMLFileTo(GroundModification.mapXMLId, map_file);
    end
    saveDensityMapToFile(GroundModification.terrDetail, density_file);
end

function GroundModification.updateGroundModifierArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, groundValue)
    local x,z, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(GroundModification.terrDetail, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ);
    setDensityParallelogram(GroundModification.terrDetail,  x, z, widthX, widthZ, heightX, heightZ, 0, 12, groundValue);
end

function GroundModification.updatePloughArea(startWorldX, superFunc, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, createField, limitGrasToField, angle)
    if createField then
        local x,z, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(GroundModification.terrDetail, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ);
        setDensityParallelogram(GroundModification.terrDetail,  x, z, widthX, widthZ, heightX, heightZ, 0, 12, 2048);
    end
    return superFunc(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, createField, limitGrasToField, angle);
end

CareerScreen.saveSavegame = Utils.appendedFunction(CareerScreen.saveSavegame, GroundModification.saveSavegame);
Utils.updatePloughArea = Utils.overwrittenFunction(Utils.updatePloughArea, GroundModification.updatePloughArea);

addModEventListener(GroundModification);

print("---- GroundModification loaded.");